{/*
   UPuissance4GameProcess.pas

   DATE   : 07/01/2007
   NOTE   : Cette unit englobe toutes les oprations concernant la gestion d'une
            partie de puissance 4
   AUTEUR : KHELIL RADOUANE ABDESSALAM
   EMAIL  : rkhelil@gmail.com
*/}

UNIT UPuissance4GameProcess;

INTERFACE
      // Permet de gerer la pile des coups jous
      Procedure Push( nColumn : Byte );
      Procedure Pop;
      Procedure Undo;
      Procedure Redo;

      procedure InitGame;
      Procedure Play( AlterStack : Boolean );

IMPLEMENTATION
Uses UPuissance4Globals;
{/*
   Retourne la valeur Max de 2 nombres.
*/}
Function Max( nVal1 , nVal2 : Integer ):Integer;
Begin
  IF ( nVal1 >= nVal2 ) Then Result := nVal1
                        Else Result := nVal2;
End;

{/*
   Initialisation de la pile
*/}
Procedure InitStack;
Var
  I : Byte;
Begin
  // Initialisation de la pile et du pointeur de la pile des coups
  For I:=1 To MAX_COL*MAX_LIN Do
    aStack[I] := 0;
  nStackPtr := 0;
End;

{/*
   Initialisation des variables pour une nouvelle partie.
*/}
Procedure InitGameWork;
Var
  I,J : Byte;
Begin
  // Initialisation du plateau du jeu
  For I:=1 To MAX_COL Do
  Begin
    aCurPtr[I] := MAX_LIN;
    For J:=1 To MAX_LIN Do
      aGame[I,J] := csEmpty;
  End;

  // Initialisations des jetons
  nBalls1    := 21;
  nBalls2    := 21;
  nCurPlayer := 0; // Joueur 1 commence
End;

{/*
   Initialisation Globale
*/}
procedure InitGame;
Begin
  InitGameWork;
  InitStack;
  nMaxMoves  := 0; // indispensable pour grer le REDO
End;

{/*
   Sauvegarde le coups jou dans la pile
*/}
Procedure Push( nColumn : Byte );
Begin
  IF ( nStackPtr < (MAX_COL*MAX_LIN)) Then
  Begin
    Inc(nStackPtr);
    aStack[nStackPtr] := nColumn;
  End;
End;

{/*
   Permet de retirer le dernier coups jou
*/}
Procedure Pop;
Begin
  IF ( nStackPtr > 0 ) Then Dec(nStackPtr);
End;

{/*
   Permet de jouer un coup et fait modifier l'etat du joueur en cours ainsi
   que l'etat des pointeurs pour connaitre la cellule libre de chaque colonne.
   AlterStack : Si elle est  TRUE en enregistre le coups sinon le coups n'est pas
                enregistr dans le cas o cette procedure est apple par UNDO et REDO.
*/}
Procedure Play( AlterStack : Boolean );
Begin
  IF ( aGame[ nSelectedColumn , aCurPtr[nSelectedColumn] ] = csEmpty ) Then
  Begin
    Case nCurPlayer Of
        // White
        0 : Begin
              IF ( nBalls1 > 0 ) Then
              Begin
                aGame[ nSelectedColumn , aCurPtr[nSelectedColumn] ] := csWhite;
                nCurPlayer := 1;
                Dec(nBalls1);
                IF AlterStack Then
                Begin
                  Push( nSelectedColumn );
                  IF (42-(nBalls1+nBalls2)) > nMaxMoves Then Inc(nMaxMoves);
                End;
              End;
            End;
        // Black
        1 : Begin
              IF ( nBalls2 > 0 ) Then
              Begin
                aGame[ nSelectedColumn , aCurPtr[nSelectedColumn] ] := csBlack;
                nCurPlayer := 0;
                Dec(nBalls2);
                IF AlterStack Then
                Begin
                  Push( nSelectedColumn );
                  IF (42-(nBalls1+nBalls2)) > nMaxMoves Then Inc(nMaxMoves);
                End;
              End;
            End;
    End;

    IF (aCurPtr[nSelectedColumn] > 1) Then
       Dec(aCurPtr[nSelectedColumn]);
  End;
End;


{/*
   Annule le dernier coups jou
*/}
Procedure Undo;
Var
  I : Integer;
Begin
  IF ( nStackPtr > 0 ) Then
  Begin
    Pop;
    InitGameWork;
    For I:=1 To nStackPtr Do
    Begin
      nSelectedColumn := aStack[I]; // Quelle colonne a t joue
      Play(FALSE);                  // On l'a joue
    End;
  End;
End;

{/*
   Rejoue le dernier coups annul
*/}
Procedure Redo;
Var
  I : Integer;
Begin
  IF ( nStackPtr < nMaxMoves ) Then
  Begin
    Inc(nStackPtr);
    InitGameWork;
    For I:=1 To nStackPtr Do
    Begin
      nSelectedColumn := aStack[I]; // Quelle colonne a t joue
      Play(FALSE);                  // On l'a joue
    End;
  End;
End;

{/*
  Initialisation des variables
*/}
BEGIN
  cPlayer1 := 'Joueur 1';
  cPlayer2 := 'Joueur 2';
END.
