{/*
   UPuissance4Main.pas

   DATE   : 08/01/2007
   NOTE   : C'est l'unit principale du programme.
   AUTEUR : KHELIL RADOUANE ABDESSALAM
   EMAIL  : rkhelil@gmail.com
*/}

UNIT UPuissance4Main;

INTERFACE

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ImgList, ToolWin, ExtCtrls, StdCtrls, Buttons, jpeg;

Type
  TPuissance4Form = class(TForm)
    ToolBar1: TToolBar;
    EnabledImages: TImageList;
    NewGameBtn: TToolButton;
    PlayersNamesBtn: TToolButton;
    SaveGameBtn: TToolButton;
    QuitBtn: TToolButton;
    DisabledImages: TImageList;
    Balls: TImageList;
    UndoBtn: TToolButton;
    RedoBtn: TToolButton;
    OpenGameBtn: TToolButton;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    GameDisplay: TImage;
    WorkArea: TImage;
    ToolButton1: TToolButton;
    Panel1: TPanel;
    CountLabel1: TLabel;
    CountLabel2: TLabel;
    BallPlayer1: TImage;
    BallPlayer2: TImage;
    InfosLabel: TLabel;
    Arrow: TImage;
    procedure QuitBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure NewGameBtnClick(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure GameDisplayMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure GameDisplayClick(Sender: TObject);
    procedure PlayersNamesBtnClick(Sender: TObject);
    procedure UndoBtnClick(Sender: TObject);
    procedure RedoBtnClick(Sender: TObject);
    procedure OpenGameBtnClick(Sender: TObject);
    procedure SaveGameBtnClick(Sender: TObject);
  private
    { Dclarations prives }
    procedure MainInit;
    procedure DrawGame;
    Procedure DisplayGameState;
  public
    { Dclarations publiques }
  End;

Var
  Puissance4Form: TPuissance4Form;

IMPLEMENTATION

Uses UPuissance4Globals,UPuissance4Players,UPuissance4RWFile,UPuissance4GameProcess,UPuissance4GetGameResult;
{$R *.dfm}
Var
  // Pour redessiner le plateau vide
  R  : TRect;
  GR : TGameResult;

{/*
   Informations sur l'tat du jeu.
*/}
Procedure TPuissance4Form.DisplayGameState;
Begin
  CountLabel1.Caption := IntToStr(nBalls1);
  CountLabel2.Caption := IntToStr(nBalls2);

  Case GR Of
    grNoWinner   : Begin
                     GameDisplay.Hint := 'Partie nulle';
                     InfosLabel.Caption := GameDisplay.Hint;
                     ShowMessage( GameDisplay.Hint );
                   End;

    grPlayer1Win : Begin
                     GameDisplay.Hint := 'Partie gagne par : ' + cPlayer1;
                     InfosLabel.Caption := GameDisplay.Hint;
                     ShowMessage( GameDisplay.Hint );
                   End;

    grPlayer2Win : Begin
                     GameDisplay.Hint := 'Partie gagne par : ' + cPlayer2;
                     InfosLabel.Caption := GameDisplay.Hint;
                     ShowMessage( GameDisplay.Hint );
                   End;

    grGameInProgress : Begin
                         EnabledImages.GetBitmap(7+nCurPlayer, Arrow.Picture.Bitmap);
                         Case nCurPlayer Of
                            0 : Begin
                                  Arrow.Left := 56;
                                  GameDisplay.Hint := cPlayer1 + '  vous';
                                End;
                            1 : Begin
                                  Arrow.Left := 192;
                                  GameDisplay.Hint := cPlayer2 + '  vous';
                                End;
                         End;
                         InfosLabel.Caption := GameDisplay.Hint;
                       End;
  End;


End;

procedure TPuissance4Form.FormClose(Sender: TObject;
  var Action: TCloseAction);
Begin
  IF MessageDlg('Quitter Puissance 4 ?',mtConfirmation,[mbYes,mbNo],0) <> mrYes Then
     Action := caNone;
End;

procedure TPuissance4Form.FormShow(Sender: TObject);
Begin
  // Rectangle pour redessiner le plateau vide
  R.Top    := 0;
  R.Left   := 0;
  R.Right  := GameDisplay.Width;
  R.Bottom := GameDisplay.Height;

  DELTA_X := Width-GameDisplay.Width;
  DELTA_Y := Height-GameDisplay.Height;
  MainInit;
  DoubleBuffered := TRUE;
  BallPlayer1.Hint := 'Couleur de : ' + cPlayer1;
  BallPlayer2.Hint := 'Couleur de : ' + cPlayer2;
End;

{/*
   Dessine le plateau du jeux
*/}
procedure TPuissance4Form.DrawGame;
Var
  I,J   : Integer;
  W,H   : Integer;
Begin
  W := GameDisplay.Width Div MAX_COL;
  IF (W < DIM_X) Then W := DIM_X;

  H := GameDisplay.Height Div MAX_LIN;
  IF (H < DIM_Y) Then H := DIM_Y;

  IF REDRAW_ALL Then
  Begin
    GameDisplay.Canvas.CopyRect( R , WorkArea.Canvas , R ) ;
    REDRAW_ALL := FALSE;
  End;

 // Dessiner l'tat du jeu sur le plateau.
 For I:=1 To MAX_COL Do
   For J:=1 To MAX_LIN Do
   Begin
     IF (aGame[I,J] <> csEmpty) Then
     Begin
       IF ( aGame[I,J] = csWhite ) Then
          Balls.Draw( GameDisplay.Canvas , W*(I-1)+4 , (H*(J-1))+1 , 0 )
       Else
          Balls.Draw( GameDisplay.Canvas , W*(I-1)+4 , (H*(J-1))+1, 1 );
     End;
   End;

  // Mise  jour des boutons
  UndoBtn.Enabled     := (nStackPtr > 0);
  RedoBtn.Enabled     := (42-(nBalls1+nBalls2)) < nMaxMoves ;
  SaveGameBtn.Enabled := (nStackPtr > 0);
End;

{/*
  Initialisation de l'interface utilisateur ainsi que le jeu.
*/}
procedure TPuissance4Form.MainInit;
Begin
  InitGame;
  UndoBtn.Enabled     := False;
  RedoBtn.Enabled     := False;
  SaveGameBtn.Enabled := False;
  GR := grGameInProgress;  // Dbut de la partie donc en cours
  REDRAW_ALL := TRUE;      // Forcer le dessin complet
  DisplayGameState;
End;

procedure TPuissance4Form.QuitBtnClick(Sender: TObject);
Begin
  Close;
End;

procedure TPuissance4Form.NewGameBtnClick(Sender: TObject);
Begin
  MainInit;
  Refresh;
End;

procedure TPuissance4Form.FormPaint(Sender: TObject);
Begin
  DrawGame;
End;

procedure TPuissance4Form.GameDisplayMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
Var
  W : Integer;
Begin
  W := GameDisplay.Width Div MAX_COL;
  nSelectedColumn :=  (X Div W) + 1;
  IF ( nSelectedColumn > 7 ) Then nSelectedColumn := 7;
End;

procedure TPuissance4Form.GameDisplayClick(Sender: TObject);
Begin
  IF (GR = grGameInProgress) Then
  Begin
    Play(TRUE);
    Refresh;
    GR := GetGameResult;
  End;
  DisplayGameState;
End;

procedure TPuissance4Form.PlayersNamesBtnClick(Sender: TObject);
Begin
  With PlayersForm Do
  Begin
    Edit1.Text := cPlayer1;
    Edit2.Text := cPlayer2;
    ShowModal;
    IF Accept Then
    Begin
      cPlayer1 := Edit1.Text;
      cPlayer2 := Edit2.Text;

      BallPlayer1.Hint := 'Couleur de : ' + cPlayer1;
      BallPlayer2.Hint := 'Couleur de : ' + cPlayer2;
      DisplayGameState;
    End;
  End;
End;

procedure TPuissance4Form.UndoBtnClick(Sender: TObject);
Begin
  Undo;
  REDRAW_ALL := TRUE; // Dessiner  nouveau 
  Refresh;
  GR := GetGameResult;
  DisplayGameState;
End;

procedure TPuissance4Form.RedoBtnClick(Sender: TObject);
Begin
  Redo;
  Refresh;
  GR := GetGameResult;
  DisplayGameState;
End;

procedure TPuissance4Form.OpenGameBtnClick(Sender: TObject);
Begin
  With OpenDialog1 Do
    IF Execute Then
    Begin
      LoadGameFromFile( FileName );
      REDRAW_ALL := TRUE; // Dessiner  nouveau
      Refresh;
      GR := GetGameResult;
      BallPlayer1.Hint := 'Couleur de : ' + cPlayer1;
      BallPlayer2.Hint := 'Couleur de : ' + cPlayer2;
      DisplayGameState;
    End;
End;

procedure TPuissance4Form.SaveGameBtnClick(Sender: TObject);
Begin
  With SaveDialog1 Do
  Begin
    FileName := 'GAME.PW4';
    IF Execute Then SaveGameToFile( FileName );
  End;
End;

END.
